<?php
/**
 * @version    5.2.1
 * @package    Com_Gafinance
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gafinance\Site\Model;

// No direct access.
\defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\MVC\Model\ListModel;
use \Joomla\CMS\Helper\TagsHelper;
use \Joomla\CMS\Layout\FileLayout;
use \Joomla\Database\ParameterType;
use \Joomla\Utilities\ArrayHelper;
use \Joomla\CMS\Component\ComponentHelper;
use \GlennArkell\Component\Gafinance\Administrator\Helper\GaauditHelper;
use \GlennArkell\Component\Gafinance\Administrator\Helper\GafinanceHelper;

/**
 * Methods supporting a list of records.
 * @since  1.6
 */
class TransactionsModel extends ListModel
{
	/**
	 * Constructor.
	 * @param   array  $config  An optional associative array of configuration settings.
	 * @see        JController
	 * @since      1.6
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'ordering', 'a.ordering',
				'state', 'a.state',
				'created_by', 'a.created_by',
				'modified_by', 'a.modified_by',
				'created_date', 'a.created_date',
				'modified_date', 'a.modified_date',
				'user_id', 'a.user_id',
				'tran_type', 'a.tran_type',
				'tran_date', 'a.tran_date',
				'tran_amount', 'a.tran_amount',
				'tran_desc', 'a.tran_desc',
				'tran_file', 'a.tran_file',
				'accnt_id', 'a.accnt_id',
				'gst_amt', 'a.gst_amt',
				'comment', 'a.comment',
			);
		}

		parent::__construct($config);
	}

	/**
	 * Method to auto-populate the model state.
	 * Note. Calling getState in this method will result in recursion.
	 * @param   string  $ordering   Elements order
	 * @param   string  $direction  Order direction
	 * @return void
	 * @throws Exception
	 * @since    1.6
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		$app  = Factory::getApplication();
            
		$list = $app->getUserState($this->context . '.list');

		$ordering  = isset($list['filter_order'])     ? $list['filter_order']     : null;
		$direction = isset($list['filter_order_Dir']) ? $list['filter_order_Dir'] : null;
		if(empty($ordering)) {
			$ordering = $app->getUserStateFromRequest($this->context . '.filter_order', 'filter_order', $app->get('filter_order'));
			if (!in_array($ordering, $this->filter_fields)) {
				$ordering = "a.tran_date";
			}
			$this->setState('list.ordering', $ordering);
		}
		if(empty($direction)) {
			$direction = $app->getUserStateFromRequest($this->context . '.filter_order_Dir', 'filter_order_Dir', $app->get('filter_order_Dir'));
			if (!in_array(strtoupper($direction ?? ''), array('ASC', 'DESC', ''))) {
				$direction = "DESC";
			}
			$this->setState('list.direction', $direction);
		}

		$list['limit']     = $app->getUserStateFromRequest($this->context . '.list.limit', 'limit', $app->get('list_limit'), 'uint');
		$list['start']     = $app->input->getInt('start', 0);
		$list['ordering']  = $ordering;
		$list['direction'] = $direction;

		$app->setUserState($this->context . '.list', $list);
		//$app->input->set('list', null);   // if you use this, list always defaults to Global

        // List state information.
        parent::populateState($ordering, $direction);

        $context = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
        $this->setState('filter.search', $context);

        $state = $this->getUserStateFromRequest($this->context.'.filter.state', 'filter_state');
        $this->setState('filter.state', $state);

        $accnt_id = $this->getUserStateFromRequest($this->context.'.filter.accnt_id', 'filter_accnt_id');
        $this->setState('filter.accnt_id', $accnt_id);

        $tran_type = $this->getUserStateFromRequest($this->context.'.filter.tran_type', 'filter_tran_type');
        $this->setState('filter.tran_type', $tran_type);

        $cat_id = $this->getUserStateFromRequest($this->context.'.filter.cat_id', 'filter_cat_id');
        $this->setState('filter.cat_id', $cat_id);

	}

	/**
	 * Build an SQL query to load the list data.
	 * @return   JDatabaseQuery
	 * @since    1.6
	 */
	protected function getListQuery()
	{
        // Create a new query object.
        $db    = $this->getDbo();
        $query = $db->getQuery(true);

        // Select the required fields from the table.
        $query->select( $this->getState( 'list.select', 'DISTINCT a.*' ) );

        $query->from('#__gafinance_transactions AS a');

		// Join over the account id reference.
		$query->select('acct.accnt_name AS accnt_id_name');
		$query->join('LEFT', '#__gafinance_accounts AS acct ON acct.id=a.accnt_id');

		// Join over the cat id reference.
		$query->select('c.title AS cat_id_name');
		$query->join('LEFT', '#__categories AS c ON c.id=a.cat_id');

		// Join over the users for the checked out user.
		$query->select('uc.name AS uEditor');
		$query->join('LEFT', '#__users AS uc ON uc.id=a.checked_out');

		// Join over the created by field 'created_by'
		$query->select('created_by.name AS created_by_name');
		$query->join('LEFT', '#__users AS created_by ON created_by.id = a.created_by');

		// Join over the created by field 'modified_by'
		$query->select('modified_by.name AS modified_by_name');
		$query->join('LEFT', '#__users AS modified_by ON modified_by.id = a.modified_by');

		// Join over the user by field 'user_id'
		$query->select('u.name AS user_id_name');
		$query->join('LEFT', '#__users AS u ON u.id = a.user_id');

		// Filter by published state
        $status = $this->getState('filter.state');
		if (is_numeric($status)) {
			$query->where('a.state = ' . (int) $status);
		} elseif ($status === '*') {
			// show all and don't filter on status
		} else {
			if (!GafinanceHelper::getSpecificUser()->authorise('core.edit', 'com_gafinance')) {
				$query->where('a.state = 1');
			} else {
				$query->where('(a.state IN (0, 1))');
			}
        }

        // Filter by search in title
        $search = $this->getState('filter.search');
        if (!empty($search)) {
            if (stripos($search, 'id:') === 0) {
                $query->where('a.id = ' . (int) substr($search, 3));
            } else {
                $search = $db->Quote('%' . $db->escape($search, true) . '%');
				$query->where('( a.tran_desc LIKE ' . $search . ' OR  a.tran_ref LIKE ' . $search . ' OR u.name LIKE ' . $search . ' )');
            }
        }

        // Filter by transaction category
        $cat_id = $this->getState('filter.cat_id');
        if (!empty($cat_id)) {
            $query->where( ' a.cat_id = '.(int)$cat_id );
        }

        // Filter by transaction type
        $tran_type = $this->getState('filter.tran_type');
        if (!empty($tran_type)) {
            $query->where( ' a.tran_type = '.$db->Quote($tran_type) );
        }

        // Filter by account
        $accnt_id = $this->getState('filter.accnt_id');
        if (!empty($accnt_id)) {
            $query->where( ' a.accnt_id = '.$db->Quote($accnt_id) );
        }

        // Add the list ordering clause.
        $query->order($db->escape(' a.tran_date DESC '));

        return $query;
	}

	/**
	 * Method to get an array of data items
	 * @return  mixed An array of data on success, false on failure.
	 */
	public function getItems()
	{
		$items = parent::getItems();
		
		return $items;
	}

	/**
	 * Overrides the default function to check Date fields format, identified by
	 * "_dateformat" suffix, and erases the field if it's not correct.
	 * @return void
	 */
	protected function loadFormData()
	{
		$app              = Factory::getApplication();
		$filters          = $app->getUserState($this->context . '.filter', array());
		$error_dateformat = false;

		foreach ($filters as $key => $value) {
			if (strpos($key, '_dateformat') && !empty($value) && $this->isValidDate($value) == null) {
				$filters[$key]    = '';
				$error_dateformat = true;
			}
		}

		if ($error_dateformat) {
			$app->enqueueMessage(Text::_("COM_GAFINANCE_SEARCH_FILTER_DATE_FORMAT"), "warning");
			$app->setUserState($this->context . '.filter', $filters);
		}

		return parent::loadFormData();
	}

	/**
	 * Checks if a given date is valid and in a specified format (YYYY-MM-DD)
	 * @param   string  $date  Date to be checked
	 * @return bool
	 */
	private function isValidDate($date)
	{
		$date = str_replace('/', '-', $date);
		return (date_create($date)) ? Factory::getDate($date)->format("Y-m-d") : null;
	}

	/**
	 * Method to get all records since last balance reset for audit purposes
	 * @return  bool
	 */
	public function auditExtract($user = null)
	{
		if ($user) {

            $lang = Factory::getLanguage();
            $lang->load('com_gafinance', JPATH_ADMINISTRATOR);
			$transactions = GafinanceHelper::getAllTransSinceReset();
			$file = GaauditHelper::createExtractFile($transactions);

	        $params = ComponentHelper::getParams('com_gafinance');
	        $app = Factory::getApplication();
	        $email_audit = $params->get('email_audit');

			if ($email_audit) {

		        $noemail = substr($params->get('inv_dummyemail'), 0, 7);

		        $mailfrom	= $app->get('mailfrom');       // system email address
		        $fromname	= $app->get('fromname');       // Site name or system name
		        $recipients = array();
		        $bcc = array();
		        $bcc[] = $mailfrom;
		        $subject = Text::_('COM_GAFINANCE_AUDIT_EXTRACT');

		        if (substr($user->email,0,7) == $noemail) {
		            $body = Text::sprintf('COM_GAFINANCE_AUDIT_EXTRACT_NOEMAIL',$user->name);
		            $recipients[] = $mailfrom;
		        } else {
		            $body = Text::sprintf('COM_GAFINANCE_EMAIL_SALUTATION',$user->name);
		            $body .= Text::_('COM_GAFINANCE_AUDIT_EXTRACT_TXT');
		            $body .= '<p>'.$fromname.'</p>';
		            $recipients[] = $user->email;
		        }

		        $sent = GafinanceHelper::processEmail($file, $subject, $recipients, $body, $bcc);
	        }

			return true;
		} else {
			return false;
		}

		


	}
}
